#
# wrigley-fishing.R, 22 Dec 24
#
# Data from:
# A MODEL AND METHOD FOR MEASURING INFORMATION SYSTEM SIZE
# Clive D. Wrigley
# An Empirical Validation of Software Cost Estimation Models
# Chris F. Kemerer

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(3)

ss=read.csv("sys-size.csv")

par(mar=c(4.1, 5.1, 0.4, 0.7))
plot(ss$LOC, ss$TotalHours, log="xy", col=pal_col[2], cex=1.3,
	xlim=c(300, 4.5e5), ylim=c(30, 2e5),
	xlab="LOC", ylab="Develment hours\n")

text(7e3, 2e2, "FOCUS")

# slin_mod=glm(TotalHours ~ LOC, data=ss)
# summary(slin_mod)
# Coefficients:
#               Estimate Std. Error t value Pr(>|t|)    
# (Intercept) -253.95428  205.66090  -1.235    0.231    
# LOC            0.29619    0.05613   5.277 3.65e-05 ***
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# (Dispersion parameter for gaussian family taken to be 461316.6)
# 
#     Null deviance: 22072053  on 21  degrees of freedom
# Residual deviance:  9226332  on 20  degrees of freedom
#   (4 observations deleted due to missingness)
# AIC: 353.26

# Yes, an errors-in-variables technique should be used.
# But this is just for a rough fit.


# Fit could be better, adding log(LOC)^2 does not change things much.
slog_mod=glm(log(TotalHours) ~ log(LOC), data=ss)
summary(slog_mod)

x_bounds=exp(seq(log(100), log(10000), by=0.1))

pred=predict(slog_mod, newdata=data.frame(LOC=x_bounds))
lines(x_bounds, exp(pred), col=pal_col[1])

# Coefficients:
#             Estimate Std. Error t value Pr(>|t|)    
# (Intercept)  -4.5339     1.1624  -3.900 0.000888 ***
# log(LOC)      1.3011     0.1548   8.405 5.39e-08 ***
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# (Dispersion parameter for gaussian family taken to be 0.4265457)
# 
#     Null deviance: 38.6656  on 21  degrees of freedom
# Residual deviance:  8.5309  on 20  degrees of freedom
#   (4 observations deleted due to missingness)
# AIC: 47.592



# slog_mod=nls(log(TotalHours) ~ a+b*log(LOC)^c, data=ss, start=list(a=1.5, b=0.1, c=1.5), trace=TRUE)
# summary(slog_mod)
# 
# Formula: log(TotalHours) ~ a + b * log(LOC)^c
# 
# Parameters:
#   Estimate Std. Error t value Pr(>|t|)
# a 1.873480   1.974595   0.949    0.355
# b 0.007989   0.033053   0.242    0.812
# c 2.976284   1.736014   1.714    0.103
# 
# Residual standard error: 0.6438 on 19 degrees of freedom
# 
# Number of iterations to convergence: 17 
# Achieved convergence tolerance: 4.732e-06
#   (4 observations deleted due to missingness)

slog_mod=nls(log(TotalHours) ~ b*log(LOC)^c, data=ss, start=list(b=0.1, c=1.5), trace=TRUE)

# Parameters:
#   Estimate Std. Error t value Pr(>|t|)    
# b  0.10863    0.04943   2.198   0.0399 *  
# c  1.91578    0.22098   8.669  3.3e-08 ***
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# Residual standard error: 0.6349 on 20 degrees of freedom
# 
# Number of iterations to convergence: 6 
# Achieved convergence tolerance: 2.972e-07
#   (4 observations deleted due to missingness)

ck=read.csv("kemerer_tIII.csv")

KSLOC_bounds=exp(seq(log(30), log(500), by=0.1))

hours_per_month=152 # Taken from paper
points(ck$KSLOC*1000, ck$ManMonths*hours_per_month, col=pal_col[3], cex=1.3)

ck_mod=glm(log(ManMonths) ~ log(KSLOC), data=ck)
summary(ck_mod)

pred=predict(ck_mod, newdata=data.frame(KSLOC=KSLOC_bounds))
lines(1000*KSLOC_bounds, hours_per_month*exp(pred), col=pal_col[1])

text(2e5, 7e3, "COBOL")

dev.copy(dev=png, file="FOCUS-COBOL-LOC-Hrs.png")
dev.off()

